import { SDK_NAME } from "../../../constants/sdk-name.qwik.mjs";
import { logger, MSG_PREFIX } from "../../../helpers/logger.qwik.mjs";
import { fastClone } from "../../fast-clone.qwik.mjs";
import { set } from "../../set.qwik.mjs";
import { getFunctionArguments } from "../helpers.qwik.mjs";
import { safeDynamicRequire } from "./safeDynamicRequire.qwik.mjs";
const getSyncValName = (key) => `bldr_${key}_sync`;
const BUILDER_SET_STATE_NAME = "BUILDER_SET_STATE";
const INJECTED_IVM_GLOBAL = "BUILDER_IVM";
const REF_TO_PROXY_FN = `
var refToProxy = (obj) => {
  if (typeof obj !== 'object' || obj === null) {
    return obj;
  }
  return new Proxy({}, {
    get(target, key) {
        if (key === 'copySync') {
          return () => obj.copySync();
        }
        const val = obj.getSync(key);
        if (typeof val?.getSync === 'function') {
            return refToProxy(val);
        }
        return val;
    },
    set(target, key, value) {
        const v = typeof value === 'object' ? new ${INJECTED_IVM_GLOBAL}.Reference(value) : value;
        obj.setSync(key, v);
        ${BUILDER_SET_STATE_NAME}(key, value)
    },
    deleteProperty(target, key) {
        obj.deleteSync(key);
    }
  })
}
`;
const processCode = ({ code, args }) => {
  const fnArgs = args.map(([name]) => `var ${name} = refToProxy(${getSyncValName(name)}); `).join("");
  return `
${REF_TO_PROXY_FN}
${fnArgs}
function theFunction() {
  ${code}
}

const output = theFunction()

if (typeof output === 'object' && output !== null) {
  return JSON.stringify(output.copySync ? output.copySync() : output);
} else {
  return output;
}
`;
};
let IVM_INSTANCE = null;
let IVM_CONTEXT = null;
const setIvm = (ivm, options = {}) => {
  if (IVM_INSTANCE)
    return;
  IVM_INSTANCE = ivm;
  setIsolateContext(options);
};
const SHOULD_MENTION_INITIALIZE_SCRIPT = SDK_NAME === "@builder.io/sdk-react-nextjs" || SDK_NAME === "@builder.io/sdk-react" || SDK_NAME === "@builder.io/sdk-qwik" || SDK_NAME === "@builder.io/sdk-vue";
const getIvm = () => {
  try {
    if (IVM_INSTANCE)
      return IVM_INSTANCE;
    const dynRequiredIvm = safeDynamicRequire("isolated-vm");
    if (dynRequiredIvm)
      return dynRequiredIvm;
  } catch (error) {
    logger.error("isolated-vm import error.", error);
  }
  const ERROR_MESSAGE = `${MSG_PREFIX}could not import \`isolated-vm\` module for safe script execution on a Node server.
    
    SOLUTION: In a server-only execution path within your application, do one of the following:
  
    ${SHOULD_MENTION_INITIALIZE_SCRIPT ? `- import and call \`initializeNodeRuntime()\` from "${SDK_NAME}/node/init".` : ""}
    - add the following import: \`await import('isolated-vm')\`.

    For more information, visit https://builder.io/c/docs/integration-tips#enabling-data-bindings-in-node-environments`;
  throw new Error(ERROR_MESSAGE);
};
function setIsolateContext(options = {
  memoryLimit: 128
}) {
  if (IVM_CONTEXT)
    return IVM_CONTEXT;
  const ivm = getIvm();
  const isolate = new ivm.Isolate(options);
  const context = isolate.createContextSync();
  const jail = context.global;
  jail.setSync("global", jail.derefInto());
  jail.setSync("log", function(...logArgs) {
    console.log(...logArgs);
  });
  jail.setSync(INJECTED_IVM_GLOBAL, ivm);
  IVM_CONTEXT = context;
  return context;
}
const getIsolateContext = () => {
  return setIsolateContext();
};
const runInNode = ({ code, builder, context, event, localState, rootSetState, rootState }) => {
  const ivm = getIvm();
  const state = fastClone({
    ...rootState,
    ...localState
  });
  const args = getFunctionArguments({
    builder,
    context,
    event,
    state
  });
  const isolateContext = getIsolateContext();
  const jail = isolateContext.global;
  jail.setSync(BUILDER_SET_STATE_NAME, function(key, value) {
    set(rootState, key, value);
    rootSetState == null ? void 0 : rootSetState(rootState);
  });
  args.forEach(([key, arg]) => {
    const val = typeof arg === "object" ? new ivm.Reference(
      // workaround: methods with default values for arguments is not being cloned over
      key === "builder" ? {
        ...arg,
        getUserAttributes: () => arg.getUserAttributes()
      } : arg
    ) : null;
    jail.setSync(getSyncValName(key), val);
  });
  const evalStr = processCode({
    code,
    args
  });
  const resultStr = isolateContext.evalClosureSync(evalStr);
  try {
    const res = JSON.parse(resultStr);
    return res;
  } catch (_error) {
    return resultStr;
  }
};
export {
  runInNode,
  setIvm
};
